import urljoin from 'url-join';
import { Utils } from '@nativescript/core';
export const fileBody = (fileName, mediaType = 'application/octet-stream') => ({
    type: 'file',
    fileName,
    mediaType,
});
export const textBody = (text, mediaType = 'text/plain') => ({
    type: 'text',
    text,
    mediaType,
});
export const jsonBody = (value, mediaType = 'application/json') => ({
    type: 'json',
    value,
    mediaType,
});
export const bytesBody = (value, mediaType = 'application/octet-stream') => ({
    type: 'bytes',
    value,
    mediaType,
});
export const formValuePart = (name, value) => ({
    type: 'form-value',
    name,
    value,
});
export const formFilePart = (name, fileName, body) => ({
    type: 'form-file',
    name,
    fileName,
    body,
});
export const part = (name, body, headers) => ({
    type: 'part',
    name,
    body,
    headers,
});
export const multiPartBody = (parts, mediaType = 'multipart/form-data') => ({
    type: 'multipart',
    parts,
    mediaType,
});
export const loopHeaders = (headers, callback) => {
    for (const [key, value] of Object.entries(headers)) {
        if (typeof value === 'string') {
            callback(key, value);
        }
        else {
            for (const val of value) {
                callback(key, val);
            }
        }
    }
};
export const printHeaders = (headers, indentation = '') => {
    if (Utils.isNullOrUndefined(headers))
        return 'NO HEADERS';
    let result = '';
    const append = (line) => {
        if (result.length > 0)
            result += '\n';
        result += indentation;
        result += line;
    };
    loopHeaders(headers, (name, value) => append(`${name}: ${value.slice(0, 60)}`));
    return result || 'NO HEADERS';
};
export const printContentSummary = (body) => {
    if (Utils.isNullOrUndefined(body))
        return 'NO BODY';
    let result = '';
    let indentation = '';
    const append = (line) => {
        if (result.length > 0)
            result += '\n';
        result += indentation;
        result += line;
    };
    const indent = () => (indentation += '    ');
    const unIndent = () => (indentation = indentation.substring(0, indentation.length - 4));
    const loopPart = (p) => {
        switch (p.type) {
            case 'form-value':
                append(`FORM-VALUE: ${p.name} = ${p.value.slice(0, 60)}`);
                break;
            case 'form-file':
                append(`FORM-FILE: ${p.name} [${p.fileName}] with body:`);
                indent();
                loop(p.body);
                unIndent();
                break;
            case 'part':
                append(`PART: `);
                indent();
                printHeaders(p.headers, indentation);
                loop(p.body);
                unIndent();
                break;
        }
    };
    const loop = (b) => {
        switch (b.type) {
            case 'bytes':
                append(`BYTES: ${b.mediaType} with ${b.value.length} bytes`);
                break;
            case 'text':
                append(`TEXT: ${b.mediaType} "${b.text.slice(0, 60)}" [${b.text.length} chars]`);
                break;
            case 'file':
                append(`FILE: ${b.mediaType} from ${b.fileName}`);
                break;
            case 'json': {
                const text = JSON.stringify(b.value);
                append(`JSON: ${text.slice(0, 60)} [${text.length} chars]`);
                break;
            }
            case 'multipart':
                append(`MULTIPART: ${b.mediaType} with ${b.parts.length} parts`);
                indent();
                for (const p of b.parts) {
                    loopPart(p);
                }
                unIndent();
                break;
            default:
                break;
        }
    };
    loop(body);
    return result || 'NO BODY';
};
export const NOOP = () => {
    // No operation
};
export const getLogger = (log) => log !== null && log !== void 0 ? log : NOOP;
export function forRelativeUrl(relativeUrl, http) {
    const request = (options) => {
        const newUrl = urljoin(relativeUrl, options.url);
        const newRequest = Object.assign(Object.assign({}, options), { url: newUrl });
        return http.request(newRequest);
    };
    return { request };
}
export class HttpError extends Error {
    constructor(message) {
        super(message);
        this.name = this.constructor.name;
    }
}
export class TransientError extends HttpError {
    constructor(message) {
        super(message);
        this.name = this.constructor.name;
    }
}
export const getResponseError = (statusCode) => {
    if (statusCode >= 500) {
        // It is a server error, so it can be transient. A retry can fix it
        return new TransientError(`error.server`);
    }
    else if (statusCode >= 400) {
        // It is a client error, so retrying wont fix the problem.
        return new HttpError(`error.client.${statusCode}`);
    }
    else if (statusCode >= 300 || statusCode < 200) {
        // It is an unexpected protocol behavior, so retrying wont fix the problem.
        return new HttpError(`error.protocol`);
    }
    return null;
};
//# sourceMappingURL=common.js.map